import typing


class Kst(object):

    """
    
    TODO: THIS CLASS IS NOT USED
    
    temperature stress coefficients

    **Attributes:**\n


    `PolH` : `float` : heat stress

    `PolC` : `float` : cold stress


    """

    def __init__(self):
        self.PolH = 1.0
        self.PolC = 1.0
